use crate::backend::{Backend, Render, RenderResult, TargetBackend};
use crate::view::{GetTemplate, Template};
use std::any::Any;

pub trait Node<B: Backend>: Render<B> {}

pub trait IntoNode<B: Backend>: Sized {
    type Node: Node<B>;

    fn into_node(self) -> Self::Node;
}

pub trait CustomComponent<B: Backend = TargetBackend>: Any + Send + GetTemplate<B> {}

pub struct CustomComponentWrapper<B: Backend, C: CustomComponent<B>> {
    component: C,
    template: Template<B>,
}

impl<B: Backend, C: CustomComponent<B>> CustomComponentWrapper<B, C> {
    pub fn new(component: C) -> Self {
        let template = component.get_template();
        Self {
            component,
            template,
        }
    }
}

impl<B: Backend, C: CustomComponent<B>> Render<B> for CustomComponentWrapper<B, C> {
    fn render(&self, context: &mut B::Context) -> RenderResult<B> {
        self.template.render(context)
    }
}

impl<B: Backend, C: CustomComponent<B>> Node<B> for CustomComponentWrapper<B, C> {}
