use uwui_dsl::NativeComponent;

use crate::backend::{Backend, TargetBackend};
use crate::layout::Gravity;
use crate::view::{Construct, ConstructGroup, View};

#[derive(Default)]
pub struct StackParams {
    pub gravity: Gravity,
    pub homogay: bool,
}

impl From<(Gravity,)> for StackParams {
    fn from(value: (Gravity,)) -> Self {
        Self {
            gravity: value.0,
            ..Default::default()
        }
    }
}

macro_rules! stack_impl {
    ($name:ident) => {
        #[derive(NativeComponent)]
        pub struct $name<B: Backend = TargetBackend> {
            children: Vec<View<B>>,
            params: StackParams,
        }

        impl<B: Backend> Construct for $name<B> {
            type Params = StackParams;

            fn construct(params: Self::Params) -> Self {
                Self {
                    params,
                    children: Vec::new(),
                }
            }
        }

        impl<B: Backend> ConstructGroup<B> for $name<B> {
            fn construct_group<I>(params: Self::Params, children: I) -> Self
            where
                I: Iterator<Item = View<B>>,
            {
                Self {
                    params,
                    children: children.collect(),
                }
            }
        }
    };
}

stack_impl!(HStack);
stack_impl!(VStack);
